import numpy as np
from polimorfo.utils import maskutils
import pytest


def test_mask_to_polygons():
    mask = np.array(
        [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 1, 1, 1, 1, 0, 0], [0, 0, 0, 0, 0, 1, 1, 1, 0, 0],
         [0, 0, 0, 0, 1, 1, 1, 1, 0, 0], [0, 0, 0, 0, 0, 1, 1, 1, 0, 0],
         [0, 0, 0, 0, 1, 1, 1, 1, 0, 0], [0, 0, 0, 0, 0, 1, 1, 1, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]],
        dtype=np.uint8)

    polygons = maskutils.mask_to_polygon(mask)
    assert len(polygons) == 1


def test_bbox():
    polygons = [[1, 2, 2, 3, 5, 5, 10, 25]]
    bbox = maskutils.bbox(polygons, 64, 64)
    area = maskutils.area(bbox)
    assert area == 4


def test_bbox_zero_area():
    polygons = [[1, 1, 2, 2, 3, 3, 4, 4]]
    bbox = maskutils.bbox(polygons, 64, 64)
    area = maskutils.area(bbox)
    assert area == 0


def test_bbox_invalid_polygons():
    with pytest.raises(Exception) as ex:
        polygons = [[]]
        maskutils.bbox(polygons, 64, 64)
        assert ex.value == 'input type is not supported.'

    with pytest.raises(Exception) as ex:
        polygons = [[1, 1, 2, 2]]
        maskutils.bbox(polygons, 64, 64)
        assert ex.value == "Argument 'bb' has incorrect type (expected numpy.ndarray, got list)"


import tarfile
import zipfile
import os
import hashlib
import shutil
import math
import re
from pathlib import Path
from typing import Tuple, List
import requests
from tqdm.autonotebook import tqdm
from argparse import ArgumentParser
import polimorfo

CACHE_DIR = polimorfo.__cache_dir__
CACHE_SUBDIR = 'datasets'


def download_file(name: str,
                  url: str,
                  file_hash=None,
                  extract: bool = True,
                  cache_dir: str = CACHE_DIR,
                  cache_subdir: str = CACHE_SUBDIR) -> List[Path]:
    """Downloads a file frolsm a URL if it not already saved
    Arguments:
        name {str} -- the name of the file (e.g. )
        url {str} -- the url of the file or the idx of the file in
            case files are download from google drive
    Keyword Arguments:
        file_hash {str} -- the hash of the file downloads (default: {None})
        extract {bool} -- try to extract the file (default: {True})
        cache_dir {str} -- the default folder where the file is saved
            (default: {carambola.utils.datautils.CACHE_DIR})
        cache_subdir {str} -- the subdir where the file is downloaded
            (default: {carambola.utils.datautils.CACHE_SUBDIR})
    """

    if cache_dir is None:
        cache_dir = CACHE_DIR

    if extract:
        assert name.endswith(tuple(
            ['zip', 'tar',
             'tar.gz'])), 'selected extract=True with a non archive file'

    cache_subdir = Path(cache_dir) / cache_subdir
    cache_subdir = cache_subdir.expanduser()
    cache_subdir.mkdir(parents=True, exist_ok=True)
    # print('saving data in path {}'.format(cache_subdir.absolute()))
    file_path = cache_subdir / name

    download = False
    if file_path.exists():
        if not validate_file(file_path, file_hash):
            download = True
    else:
        download = True

    if download:
        if url.startswith('https://drive.google.com'):
            download_from_gdrive(url, file_path)
        elif url.startswith('http'):
            download_url(url, file_path)
        else:
            download_from_gdrive(url, file_path)

    if extract:
        return extract_archive(file_path, cache_subdir)

    return [file_path]


def _hash_file(fpath, algorithm='sha256', chunk_size=65535):
    """Calculates a file sha256 or md5 hash.
        Example:
        ```python
        _hash_file('/path/to/file.zip')
        'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
        ```
        Arguments:
            fpath: path to the file being validated
            algorithm: hash algorithm, one of `'auto'`, `'sha256'`, or `'md5'`.
                The default `'auto'` detects the hash algorithm in use.
            chunk_size: Bytes to read at a time, important for large files.
        Returns:
            The file hash
  """
    if (algorithm == 'sha256') or (algorithm == 'auto' and len(hash) == 64):
        hasher = hashlib.sha256()
    else:
        hasher = hashlib.md5()

    with open(fpath, 'rb') as fpath_file:
        for chunk in iter(lambda: fpath_file.read(chunk_size), b''):
            hasher.update(chunk)

    return hasher.hexdigest()


def validate_file(fpath, file_hash, algorithm='auto', chunk_size=65535):
    """Validates a file against a sha256 or md5 hash.
        Arguments:
            fpath: path to the file being validated
            file_hash:  The expected hash string of the file.
                The sha256 and md5 hash algorithms are both supported.
            algorithm: Hash algorithm, one of 'auto', 'sha256', or 'md5'.
                The default 'auto' detects the hash algorithm in use.
            chunk_size: Bytes to read at a time, important for large files.
        Returns:
            Whether the file is valid
  """
    if (algorithm == 'sha256') or (algorithm == 'auto'
                                   and len(file_hash) == 64):
        hasher = 'sha256'
    else:
        hasher = 'md5'

    if str(_hash_file(fpath, hasher, chunk_size)) == str(file_hash):
        return True
    else:
        return False


def __get_confirm_token(response):
    for key, value in response.cookies.items():
        if key.startswith('download_warning'):
            return value
    return None


def __save_response_content(response: requests.Response, destination: Path):
    """save the content of the request as file
    Arguments:
        response {requests.Response} -- the response to download
        destination {Path} -- the destination path
    """
    chunk_size = 32768

    num_elems = math.ceil(256483864 / chunk_size)

    with open(destination, "wb") as fh:
        for chunk in tqdm(response.iter_content(chunk_size),
                          total=num_elems,
                          desc='saving file {}'.format(destination.name)):
            if chunk:  # filter out keep-alive new chunks
                fh.write(chunk)


def download_from_gdrive(uri: str, dst_path: str) -> Path:
    """download a file/folder from google drive
    Given a url https://drive.google.com/file/d/1EcUzQPNQXQGiHES9gU7oh-886wbBH3VF/view?usp=sharing
    the idx -> 1EcUzQPNQXQGiHES9gU7oh-886wbBH3VF
    Arguments:
        uri {str} -- the id of the file to download or the full google drive url
        dst_path {str} -- the path to save the file
    Returns:
        Path -- the path where the file is saved
    """
    gdrive_resource_re = r"https:\/\/drive\.google.com\/file\/d\/([\w-]+)\/?"
    regex_match = re.match(gdrive_resource_re, uri.strip())
    if regex_match:
        idx = regex_match.groups()[0]
    else:
        idx = uri

    dst_path.parent.mkdir(exist_ok=True, parents=True)

    base_url = "https://docs.google.com/uc?export=download"

    with requests.Session() as session:
        response = session.get(base_url, params={'id': idx}, stream=True)
        token = __get_confirm_token(response)

        if token:
            params = {'id': idx, 'confirm': token}
            response = session.get(base_url, params=params, stream=True)

        __save_response_content(response, dst_path)

    dst_path = Path(dst_path)
    return dst_path


def download_url(url: str, dst_path: str) -> Tuple[Path, int]:
    """download a url to the destination folder
    Arguments:
        url {str} -- [description]
        dst_path {str} -- [description]
    Returns:
        Tuple[Path, int] -- [description]
    """
    dst_path = Path(dst_path)

    req = requests.get(url, stream=True)
    file_size = int(req.headers['Content-Length'])
    chunk = 1
    chunk_size = 1024
    num_bars = int(file_size / chunk_size)

    with open(dst_path, 'wb') as fp:
        for chunk in tqdm(req.iter_content(chunk_size=chunk_size),
                          total=num_bars,
                          unit='KB',
                          desc='downloading {}'.format(dst_path.name),
                          leave=True):
            fp.write(chunk)
    return file_size


def extract_archive(file_path: str,
                    dst_path: str = '',
                    archive_format='auto') -> List[Path]:
    """Extract the archive if it match tar, tar.gz, tar.bz or zip format
    Arguments:
        file_path {str} -- the path to the archive
    Keyword Arguments:
        dst_path {str} -- the path to extract the folder (default: {None}
            the directory where the archive is placed)
        archive_format {str} -- The format of the archive (default: {'auto'})
            Options are: 'auto', 'tar', 'zip'
    Returns:
        List[Path] -- the paths where the file is saved
    """
    file_path = Path(file_path)
    if dst_path == '':
        dst_path = file_path.parent
    else:
        dst_path = Path(dst_path)
    dst_path.mkdir(exist_ok=True, parents=True)
    dst_path = dst_path / file_path.name.split('.')[0]

    if archive_format == 'auto':
        archive_format = ['tar', 'zip']
    if isinstance(archive_format, str):
        archive_format = [archive_format]

    for archive_type in archive_format:
        if archive_type == 'tar':
            open_fn = tarfile.open
            is_match_fn = tarfile.is_tarfile
        elif archive_type == 'zip':
            open_fn = zipfile.ZipFile
            is_match_fn = zipfile.is_zipfile

        if is_match_fn(file_path):
            with open_fn(file_path) as fh:
                try:
                    fh.extractall(dst_path)
                except (tarfile.TarError, RuntimeError, KeyboardInterrupt):
                    if dst_path.exists():
                        if dst_path.is_file():
                            os.remove(dst_path)
                        else:
                            shutil.rmtree(dst_path)
                    raise
            list_files = list(dst_path.iterdir())
            # flatten the structure in the case the folder contains just one folder
            if (len(list_files) == 1) and (list_files[0].name
                                           == file_path.name.split('.')[0]):
                parent = list_files[0].parent
                # copy the content of the file in the original folder
                for file in list_files[0].iterdir():
                    shutil.move(file.as_posix(), parent.as_posix())
                shutil.rmtree(list_files[0].as_posix())

                return [parent]

            return list(dst_path.iterdir())
    # return the input as output
    return [file_path]


if __name__ == '__main__':
    parser = ArgumentParser("compute sha256 of a given file")
    parser.add_argument('--file', type=Path, required=True)

    args = parser.parse_args()
    print(_hash_file(args.file))



from pathlib import Path
from polimorfo.datasets.coco import CocoDataset
from typing import Dict, List, Tuple
from tqdm import tqdm
import numpy as np
from . import maskutils
import pandas as pd
from sklearn import metrics
import sys

__all__ = [
    'generate_predictions', 'mean_average_precision_and_recall',
    'mean_average_precision_and_recall_per_class'
]

REPORT_HEADER = [
    'img_path', 'gt_ann_id', 'pred_ann_id', 'true_class_id', 'pred_class_id',
    'intersection', 'union', 'IOU', 'score', 'true_area', 'pred_area',
    'class_value'
]


def __best_match(pred_anns: List, gt_img_meta: Dict, gt_ann_id: int,
                 gt_mask: np.ndarray, img_path: str, gt_class_id: int,
                 gt_area: int) -> Tuple[int, List]:
    """
    compute the best prediction given the ground truth annotation

    Args:
        pred_anns (List): the list of the annotations for the image
        gt_img_meta (Dict): the metadata of the img
        gt_ann_id (int): the idx of the grount truth annotation
        gt_mask (np.ndarray): the mask for the ground truth
        img_path (str): the path of the image
        gt_class_id (int): the idx of the ground truth class
        gt_area: the area of the gt annotation

    Returns:
        Tuple[int, List]: the id of the best prediction and the values to be saved
    """
    best_pred_ann_id = -1
    best_iou = 0

    # false negative as default
    best_values = [
        img_path, gt_ann_id, -1, gt_class_id, 0, 0, 0, 0, 1, gt_area,
        sys.float_info.max, 'false_negative'
    ]
    for pred_ann in pred_anns:
        pred_mask = maskutils.polygons_to_mask(pred_ann['segmentation'],
                                               gt_img_meta['height'],
                                               gt_img_meta['width'])
        pred_ann_id = pred_ann['id']
        pred_class_id = pred_ann['category_id']
        pred_score = pred_ann['score'] if 'score' in pred_ann else 1

        intersection = (pred_mask * gt_mask).sum()
        union = np.count_nonzero(pred_mask + gt_mask)
        iou = intersection / union

        if iou > best_iou:
            if pred_class_id == gt_class_id:
                value = 'true_positive'
            else:
                value = 'false_positive'

            best_values = [
                img_path, gt_ann_id, pred_ann_id, gt_class_id, pred_class_id,
                intersection, union, iou, pred_score, gt_area, pred_ann['area'],
                value
            ]
            best_pred_ann_id = pred_ann_id
            best_iou = iou
    return best_pred_ann_id, best_values


def generate_predictions_from_ds(gt_ds, pred_ds) -> pd.DataFrame:
    gt_ds.reindex()
    pred_ds.reindex()

    results = []

    for img_idx, gt_img_meta in tqdm(gt_ds.imgs.items()):
        gt_anns = gt_ds.get_annotations(img_idx)
        pred_img_meta = pred_ds.imgs[img_idx]

        if gt_img_meta['file_name'] != pred_img_meta['file_name']:
            raise Exception("images path compared are different")

        img_path = gt_img_meta['file_name']

        pred_anns = pred_ds.get_annotations(img_idx)
        # create a set with all the prediction that will be used to find FP
        pred_idx_dict = {ann['id']: ann for ann in pred_anns}

        if (len(gt_anns) == 0) and (len(pred_anns) == 0):
            results.append([
                img_path, -1, -1, -1, -1, 0, 0, 0, 1, sys.float_info.max,
                sys.float_info.max, 'true_negative'
            ])

        # iterate of the gr annotations
        for gt_ann in gt_anns:
            gt_mask = maskutils.polygons_to_mask(gt_ann['segmentation'],
                                                 gt_img_meta['height'],
                                                 gt_img_meta['width'])
            gt_ann_id = gt_ann['id']
            gt_class_id = gt_ann['category_id']

            pred_ann_id, row = __best_match(pred_anns, gt_img_meta, gt_ann_id,
                                            gt_mask, img_path, gt_class_id,
                                            gt_ann['area'])
            results.append(row)
            if pred_ann_id in pred_idx_dict:
                del pred_idx_dict[pred_ann_id]
                pred_anns = pred_idx_dict.values()

        # add the false positive
        for pred_ann_id, pred_ann in pred_idx_dict.items():
            #put a false positive with high score in order to not remove it from metrics
            results.append([
                img_path, -1, pred_ann_id, 0, pred_ann['category_id'], 0, 0, 0,
                pred_ann['score'], 0, pred_ann['area'], 'false_positive'
            ])

    return pd.DataFrame(results, columns=REPORT_HEADER)


def generate_predictions(gt_path: str,
                         preds_path: str,
                         images_path: str = None,
                         **kwargs) -> pd.DataFrame:
    """
    create a list that contains the comparison between the predictions
        and the ground truth to be used to compute all the metrics

    Args:
        gt_path (str): the path of the ground truth annotations
        preds_path (str): the path of the prediction annotations
        images_path (str): the path were are saved the images

    Raises:
        Exception: returns an execption if the image idx of the files are not aligned

    Returns:
        pd.DataFrame: [description]
    """
    if images_path is None:
        images_path = Path(gt_path).parent / 'images'

    gt_ds = CocoDataset(gt_path, images_path)
    gt_ds.reindex()
    pred_ds = CocoDataset(preds_path, images_path)
    pred_ds.reindex()
    return generate_predictions_from_ds(gt_ds, pred_ds)


def mean_average_precision_and_recall(
        prediction_report: pd.DataFrame,
        range_iou: np.ndarray = np.arange(.5, 1., .05),
        min_score: float = 0.0) -> Tuple[float, float]:
    """
    compute mean average precision and recall for a given range

    Args:
        prediction_report (pd.DataFrame): a dataframe generated using the method
            :func:`generate_predictions <polimorfo.utils.cocoeval.generate_predictions>`
        range_iou (np.ndarray, optional): a valid range of values. Defaults to np.arange(.5, 1., .05).
        min_score: (float): the min score used to filter annotations. Defaults: 0.0

    Returns:
        Tuple[float, float]: mean average precision and mean average recall
    """

    df = prediction_report
    # filter low score predictions
    df = df[df['score'] >= min_score]
    precisions = []
    recalls = []
    for iou in range_iou:
        true_positives = len(df[(df['true_class_id'] == df['pred_class_id']) &
                                (df['IOU'] >= iou)])
        # all the prediction that do not have a valid gt annotation
        false_positives = len(df[df['gt_ann_id'] == -1])
        # all the gt annotations that do not have a prediction
        false_negatives = len(df[df['pred_ann_id'] == -1])
        precisions.append(true_positives / (true_positives + false_positives))
        recalls.append(true_positives / (true_positives + false_negatives))

    if len(precisions) == 0:
        return None, None

    if len(precisions) == 1:
        return precisions[0], recalls[0]

    return np.mean(precisions), np.mean(recalls)


def mean_average_precision_and_recall_per_class(
        prediction_report: pd.DataFrame,
        range_iou: np.ndarray = np.arange(.5, 1., .05),
        min_score: float = 0.0,
        idx_class_dict: Dict[int,
                             str] = None) -> Dict[int, Tuple[float, float]]:
    """
    generate mean average precision and recall for class idx

    Args:
        prediction_report (pd.DataFrame): a dataframe generated using the method
            :func:`generate_predictions <polimorfo.utils.cocoeval.generate_predictions>`
        range_iou (np.ndarray, optional): a valid range of values. Defaults to np.arange(.5, 1., .05).

    Returns:
        Dict[int, Tuple[float, float]]: a dictionarry of shape class_id -> Tuple[MAP, MAR]
    """

    df = prediction_report
    class_idx_metrics = dict()
    class_idxs = sorted(df['true_class_id'].unique())
    for class_idx in class_idxs:
        if class_idx in [-1, 0]:
            continue
        df_class = df[(df['true_class_id'] == class_idx) |
                      (df['pred_class_id'] == class_idx)]

        map_mar = mean_average_precision_and_recall(df_class, range_iou,
                                                    min_score)
        if idx_class_dict is not None and class_idx in idx_class_dict:
            class_name = idx_class_dict[class_idx]
            class_idx_metrics[class_name] = map_mar
        else:
            class_idx_metrics[class_idx] = map_mar

    return class_idx_metrics


def precision_recall_per_image(prediction_report: pd.DataFrame,
                               image_name: str,
                               range_iou: np.ndarray = np.arange(.5, 1., .05),
                               min_score: float = 0.0) -> Tuple[float, float]:
    df = prediction_report
    df_image = df[df['img_path'] == image_name]
    return mean_average_precision_and_recall(df_image, range_iou, min_score)


def confusion_matrix(
        prediction_report: pd.DataFrame,
        min_iou,
        min_score,
        idx_class_dict: Dict[int, str] = None,
        normalize=False) -> Tuple[np.ndarray, metrics.ConfusionMatrixDisplay]:
    df = prediction_report[prediction_report['IOU'] >= min_iou]
    df = df[df['score'] >= min_score]

    class_idxs = sorted(
        list(
            set(df['true_class_id'].unique().tolist() +
                df['pred_class_id'].unique().tolist())))

    labels = []
    if idx_class_dict is not None:
        for idx in class_idxs:
            if idx == 0:
                labels.append('no_danno')
            else:
                labels.append(idx_class_dict[idx])
    else:
        labels = class_idxs

    cm = metrics.confusion_matrix(df['true_class_id'], df['pred_class_id'])
    if normalize:
        cm = cm / cm.sum()
    cm_display = metrics.ConfusionMatrixDisplay(cm, display_labels=labels)
    return cm, cm_display



#!/usr/bin/env python
"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt', 'r') as fh:
    requirements = fh.read().split('\n')

with open('requirements_dev.txt', 'r') as fh:
    test_requirements = fh.read().split('\n')

setup_requirements = [
    'pytest-runner',
]

setup(
    author="Fabio Fumarola",
    author_email='fabiofumarola@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description=
    "a dataset loader and converter for object detection segmentation and classification",
    entry_points={
        'console_scripts': ['polimorfo=polimorfo.cli:main',],
    },
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='polimorfo',
    name='polimorfo',
    packages=find_packages(include=['polimorfo', 'polimorfo.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/fabiofumarola/polimorfo',
    version='0.9.24',
    zip_safe=False,
)



import pytest
from pytest import fixture
from pathlib import Path
import shutil
from polimorfo.datasets import CocoDataset
import os
import numpy as np
from PIL import Image
from tqdm import tqdm

BASE_PATH = Path(__file__).parent.parent / 'data'


@fixture
def dataset_file():
    return BASE_PATH / 'hair_drier_toaster_bear.json'


@fixture
def coco_test(dataset_file):
    return CocoDataset(dataset_file)


def test_load_coco(dataset_file):
    coco = CocoDataset(dataset_file)

    assert len(coco.cats) == 3


def test_categories_images_count(coco_test):
    images_count = coco_test.cats_images_count()
    assert len(images_count) == 3
    print(images_count)
    assert images_count == {'toaster': 217, 'hair drier': 189, 'bear': 960}


def test_categories_annotations_count(coco_test):
    images_count = coco_test.cats_annotations_count()
    assert len(images_count) == 3
    print(images_count)
    assert images_count == {'toaster': 225, 'hair drier': 198, 'bear': 1294}


def test_keep_categories(coco_test):
    coco_test.keep_categories([80, 89], True)
    assert len(coco_test.cats) == 2


def test_dumps(coco_test):
    coco_test.keep_categories([80, 89], True)

    data = coco_test.dumps()
    assert len(data['categories']) == 2
    assert len(data['images']) == 406


def test_dump_segmentation(coco_test):
    out_path = BASE_PATH / 'segments'
    coco_test.dump(out_path, CocoDataset.ExportFormat.segmentation)
    assert len(list(out_path.glob('*.png'))) > 0
    shutil.rmtree(out_path.as_posix())


def test_save_segmentation_masks(coco_test):
    out_path = BASE_PATH / 'segments'
    coco_test.save_segmentation_masks(out_path, [23], {23: 24})
    assert len(list(out_path.glob('*.png'))) > 0
    distinct_values = set()
    for png_path in tqdm(list(out_path.glob('*.png'))):
        img = np.array(Image.open(png_path))
        distinct_values = distinct_values.union(set(np.unique(img)))

    assert distinct_values == {0, 24}
    shutil.rmtree(out_path.as_posix())


def test_create_dataset():
    annotations_path = BASE_PATH / 'new_coco.json'
    ds = CocoDataset(annotations_path)

    cat_id = ds.add_category('dog', 'animal')
    img_id = ds.add_image((BASE_PATH / 'test_nodamage.jpg').as_posix())
    ds.add_annotation(img_id, cat_id, [1, 2, 3, 4, 5], 10, [0, 0, 256, 256], 0)

    assert len(ds) == 1
    assert len(ds.anns) == 1

    ds.dump()
    assert annotations_path.exists()

    with open(annotations_path, 'r') as f:
        assert len(f.readline()) > 0

    os.remove(annotations_path)


def test_create_dataset_existing():
    ds = CocoDataset(BASE_PATH / 'new_coco.json')

    cat_id = ds.add_category('dog', 'animal')
    img_id = ds.add_image((BASE_PATH / 'test_nodamage.jpg').as_posix())
    ds.add_annotation(img_id, cat_id, [1, 2, 3, 4, 5], 10, [0, 0, 256, 256], 0)

    img_id = ds.add_image((BASE_PATH / 'test_nodamage.jpg').as_posix())
    ds.add_annotation(img_id, cat_id, [1, 2, 3, 4, 5], 10, [0, 0, 256, 256], 0)

    assert len(ds.imgs) == 2
    assert len(ds.anns) == 2

    assert ds.cat_id == 2
    assert ds.img_id == 2
    assert ds.ann_id == 2


def test_remove_categories():
    ds = CocoDataset(BASE_PATH / 'new_coco.json')
    cat_id = ds.add_category('dog', 'animal')
    assert len(ds.cats) == 1
    ds.remove_categories([cat_id])

    assert len(ds.cats) == 0


def test_remove_categories_and_annotations():
    ds = CocoDataset(BASE_PATH / 'new_coco.json')
    cat_id = ds.add_category('dog', 'animal')
    img_id = ds.add_image((BASE_PATH / 'test_nodamage.jpg').as_posix())
    ds.add_annotation(img_id, cat_id, [1, 2, 3, 4, 5], 10, [0, 0, 256, 256], 0)

    assert len(ds.cats) == 1
    ds.remove_categories([cat_id])

    assert len(ds.cats) == 0


def test_create_split(coco_test):
    train, val, test = coco_test.split(.8, .1)
    assert len(train) > 0
    assert len(val) > 0
    assert len(test) > 0


def test_create_split_notest(coco_test: CocoDataset):
    train, val, test = coco_test.split(.8, .3)
    assert len(train) > 0
    assert len(val) > 0
    assert len(test) == 0


def test_copy(coco_test: CocoDataset):
    copy_coco = coco_test.copy()
    assert id(coco_test.imgs) != id(copy_coco.imgs)
    assert id(coco_test.cats) != id(copy_coco.cats)
    assert id(coco_test.anns) != id(copy_coco.anns)


def test_update_images_path(coco_test: CocoDataset):
    coco_test.update_images_path(lambda x: Path(x).name)
    coco_test.reindex()
    assert coco_test.imgs[1]['file_name'] == '000000001442.jpg'